/*
 * C6Run_rpc.c
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/************************************************************************
*  Routines for handling Dispatching RPCs to the DSP                    *
************************************************************************/

/*  ----------------------------------- Standard Headers                */
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*  ----------------------------------- Linux Posix Headers            */
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <pthread.h>
#include <unistd.h>

/*  ----------------------------------- DSP/BIOS Link                   */
#include <dsplink.h>

/*  ----------------------------------- DSP/BIOS LINK API               */
#include <proc.h>
#include <procdefs.h>
#include <msgq.h>
#include <pool.h>
#include <loaderdefs.h>

/*  ----------------------------------- CMEM API                        */
#include <ti/sdo/linuxutils/cmem/include/cmem.h>

/*  ----------------------------------- Application Headers             */
#include "rpc_ipc.h"
#include "C6Run_mem.h"
#include "C6Run.h"

/*  ----------------------------------- This module's header            */
#include "C6Run_rpc.h"

/************************************************************
* Explicit External Declarations                            *
************************************************************/


/************************************************************
* Local Macro Declarations                                  *
************************************************************/


/************************************************************
* Local Typedef Declarations                                *
************************************************************/



/************************************************************
* Local Function Declarations                               *
************************************************************/


/************************************************************
* Local Variable Definitions                                *
************************************************************/


/************************************************************
* Global Variable Definitions                               *
************************************************************/

volatile bool C6RUN_RPC_initialized = FALSE;


/************************************************************
* Global Function Definitions                               *
************************************************************/

// C6Run RPC initialzation routine
int C6RUN_RPC_init( void )
{
  int status = C6RUN_SUCCESS;

  // Return if the flag is already set
  if (C6RUN_RPC_initialized){  return C6RUN_SUCCESS;  }
  
  // Init child modules (ones we depend on)
  status |= C6RUN_libInit();
  
  // Register exit routine for calling upon program termination
  atexit( C6RUN_RPC_exit );

  // Set the flag
  C6RUN_RPC_initialized = TRUE;
  
  // Return status
  return status;
}

// C6Run RPC cleanup routine
void C6RUN_RPC_exit( void )
{
  C6RUN_RPC_initialized = FALSE;
}

// Send function call message and await reply indicating completion
void C6RUN_RPC_dispatch(void *msg)
{
  uint16_t msgId;
  
  RPC_IPC_sendMsg(msg,RPC_GPP2DSP_FXNCALL);
  
  msgId = RPC_IPC_recvMsg( (RPC_Msg **) &msg );
  
  if (msgId != RPC_GPP2DSP_FXNCALL_RET)
  {
    printf("MSGQ_getMsgId is not RPC_GPP2DSP_FXNCALL_RET.");
    exit(1);
  }
}

void* C6RUN_RPC_allocMsgBuff(void **msg)
{
  RPC_Msg **rpcMsg = (RPC_Msg **)msg;
  RPC_IPC_allocMsg( rpcMsg );

  return (void *) ((*rpcMsg)->buffer);
}

void C6RUN_RPC_freeMsgBuff(void *msg)
{
  RPC_IPC_freeMsg((RPC_Msg *) msg);
}


/***********************************************************
* Local Function Definitions                               *
***********************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/
